@@ -29,7 +29,6 @@ import mock
 from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_config import fixture as config_fixture
-import oslo_i18n
 import oslo_messaging
 from oslo_messaging import conffixture as messaging_conffixture
 from oslo_utils import uuidutils
@@ -49,6 +48,9 @@ test_opts = [
     cfg.StrOpt('sqlite_clean_db',
                default='clean.sqlite',
                help='File name of clean sqlite database.'),
+    cfg.StrOpt('sqlite_db',
+               default='manila.sqlite',
+               help='The filename to use with sqlite.'),
 ]
 
 CONF = cfg.CONF
@@ -87,7 +89,7 @@ class Database(fixtures.Fixture):
         if self.sql_connection == "sqlite://":
             conn = self.engine.connect()
             conn.connection.executescript(self._DB)
-            self.addCleanup(self.engine.dispose)  # pylint: disable=E1101
+            self.addCleanup(self.engine.dispose)  # pylint: disable=no-member
         else:
             shutil.copyfile(
                 os.path.join(CONF.state_path, self.sqlite_clean_db),
@@ -108,7 +110,6 @@ class TestCase(base_test.BaseTestCase):
         """Run before each test method to initialize test environment."""
         super(TestCase, self).setUp()
 
-        oslo_i18n.enable_lazy(enable=False)
         conf_fixture.set_defaults(CONF)
         CONF([], default_config_files=[])
 
